; 					NESmas '03
; 					/Mic, 2003
;
;   		    http://nangijala.no-ip.org  |  stabmaster_@hotmail.com


; Assemble with NESASM 2.51



 .inesprg    2			; Two 16k PRG-ROM banks
 .ineschr    1			; One 8k CHR-ROM bank
 .inesmir    1			; Vertical mirroring
 .inesmap    0			; Mapper 0 (none)



; Variables. Just picked a random RAM address not used by the MCK driver.
 .org $0610
 	flipflop: 	.db 0	; Used to slow down scrolling a bit
 	msg_nt: 	.db 0	; Nametable where the scrolling message is shown (0=$2000, 1=$2400)
 	msg_hi: 	.db 0	; High byte of the position in the message string (which can be >255)
 	msg_pos: 	.db 0	; Low byte -- " --
 	counter:	.db 0	; Used as a counter when both X & Y are needed for other things
 	hscroll: 	.db 0	; Horizontal scroll value
 	adrlo:		.db 0
 	adrhi:		.db 0
 	
 	

 .bank 0
 .org  $8000
 
	; Sound data
	.include	"n106.h"
	.include 	"fds.h"
	.include	"sounddrv.h"
	.include	"freqdata.h"
	.include	"effect.h"
	.include	"songdata.h"



	
 .bank 2
 .org $C000			


reset:
	sei
        cld
        ldx #$ff
        txs

	jsr setup
	
	jsr copy_gfx

	lda #$0B
	cli
	jsr sound_init
	
	jsr enable_screen
	
	forever:
		wait_sprite_clear:
			bit $2002
			bvs wait_sprite_clear
		wait_sprite_hit:
			bit $2002
			bvc wait_sprite_hit
		
		; Set BG pattern table address to $0000
		lda #$88
		sta $2000
		
		; Reset scrolling
        	lda #$03       
		sta $2005
		lda #$00
		sta $2005
		
		jmp forever



setup:		
	; Wait for vblank
	w_vbi:
		lda $2002
	        bpl w_vbi

	; Disable NMI
	lda #$00
	sta $2000

	; Disable the screen for now
	lda #$00
	sta $2001  
	
	; Reset scrolling registers
	sta $2005
	sta $2005
	
	tax
	clear_ram:
		sta $0000,X
		; Don't clear page 1 since the stack is there
	        sta $0200,X
	        sta $0300,X
	        sta $0400,X
	        sta $0500,X
	        sta $0600,X
	        sta $0700,X
	        inx
	        bne clear_ram


	
	; Clear nametable 0
	lda #$20
	sta $2006
	lda #$00
	sta $2006
	ldy #$1E			; Clear 30 rows (= 240 pixels)
	lda #$FD			; Use tile 253 (which is all black)
	clear_nt:
		ldx #$20		; 32 tiles per row
		clear_row:
			sta $2007	; Write to VRAM, address is auto-incremented
			dex
			bne clear_row
		dey
		bne clear_nt



	; Clear attributetable 0
	lda #$23
	sta $2006
	lda #$C0
	sta $2006
	
	lda #$00
	ldx #$40
	clear_at:
		sta $2007	; Write to VRAM, address is auto-incremented
		dex
		bne clear_at


	
	; Setup palette
        lda #$3F       
        sta $2006      
        lda #$00        
        sta $2006 
	ldx #$00
	ldy #$20
	set_palette:
		lda palette,X        
	        sta $2007
		inx
		dey
		bne set_palette

	rts



print_message:
	lda msg_nt
	eor #1			; Invert to draw to the nametable that isn't shown atm
	asl A			; Multiply by 4
	asl A			; ...
	ora #$20		; OR with $20 to get either $20 or $24
	sta $2006
	lda #$40		; Start 2 tiles down (scanline 16)
	sta $2006

	ldy msg_pos
	lda #$20
	sta counter
	pm_loop:
		lda msg_hi	; Check if the high byte is set (ie. the position is >=256)	
		bne hi_set
			ldx message,Y
			jmp print
		hi_set:
			ldx message_b,Y
		print:
		lda character_set,X
		clc
		adc #39		; The font is located at 39-110 in the second pattern table
		sta $2007
		iny
		dec counter
		bne pm_loop

	lda msg_pos
	clc
	adc #$20
	sta msg_pos
	bne pm_no_wrap
		; Flip the high position byte if we crossed a 256-byte boundry
		lda msg_hi
		eor #1
		sta msg_hi
	pm_no_wrap:

	; Reset the VRAM pointer
	lda msg_nt
	eor #1
	asl A
	asl A
	sta $2006
	lda #$00
	sta $2006	
	
	rts
	
	

copy_gfx:
	; Setup nametable 0
	lda #$20
	sta $2006
	lda #$00
	sta $2006

	lda #37
	ldx #$20
	set_nt_d:
		sta $2007
		dex
		bne set_nt_d
	
	lda #35
	ldx #$20
	set_nt_a:
		sta $2007
		dex
		bne set_nt_a

	lda #39
	ldx #$20
	set_nt_b:
		sta $2007
		dex
		bne set_nt_b

	lda #36
	ldx #$20
	set_nt_c:
		sta $2007
		dex
		bne set_nt_c
		
	
	; ..start copying from ROM..
	ldy #$00
	ldx #$00
	copy_nt_a:
		lda nametable_a,X
		sta $2007
		inx
		dey
		bne copy_nt_a

	; ..continue..
	ldy #$00
	ldx #$00
	copy_nt_b:
		lda nametable_b,X
		sta $2007
		inx
		dey
		bne copy_nt_b

	; ..continue
	ldy #$00
	ldx #$00
	copy_nt_c:
		lda nametable_c,X
		sta $2007
		inx
		dey
		bne copy_nt_c

	; Setup attributetable 0
	lda #$23
	sta $2006
	lda #$C8
	sta $2006
	ldx #$00
	ldy #$30
	copy_at:
		lda attrtable,X
		sta $2007	
		inx
		dey
		bne copy_at


	; Setup nametable 1
	lda #$24
	sta $2006
	lda #$00
	sta $2006

	lda #37
	ldx #$20
	set_nt_e:
		sta $2007
		dex
		bne set_nt_e
	
	lda #35
	ldx #$20
	set_nt_f:
		sta $2007
		dex
		bne set_nt_f

	lda #38
	ldx #$20
	set_nt_g:
		sta $2007
		dex
		bne set_nt_g

	lda #36
	ldx #$20
	set_nt_h:
		sta $2007
		dex
		bne set_nt_h

	; Setup leftmost column of nametable 1 starting at y=160
	lda #$26
	sta adrhi
	sta $2006
	lda #$80
	sta adrlo
	sta $2006
	lda #189
	sta $2007
	
	ldy #$07
	ldx #199
	set_colm:
		lda adrlo
		clc
		adc #$20	; Jump to next row (32 tiles)
		sta adrlo
		lda adrhi
		adc #$00	; Increase hi address on carry
		sta adrhi
		sta $2006
		lda adrlo
		sta $2006
		stx $2007
		dey
		bne set_colm
		
		
	; Print the first 32 characters of the message
	jsr print_message

	; Setup attributetable 1
	lda #$27
	sta $2006
	lda #$C8
	sta $2006
	ldx #$00
	ldy #$30
	copy_at_b:
		lda attrtable,X
		sta $2007	
		inx
		dey
		bne copy_at_b
		
		
	; Setup sprites
	lda #$00
	sta $2003
	ldy #$C0
	tax
	copy_spr:
		lda sprite_data,X
		sta $2004
		inx
		dey
		bne copy_spr
		

	; Reset VRAM pointer
	lda #$00
	sta $2006
	lda #$00
	sta $2006		

	rts
	
		
		
	
	
enable_screen:
	w_vbi_e:
		lda $2002
	        bpl w_vbi_e

	; Enable BG & sprites, don't clip BG
	lda #$1A
	sta $2001  

	; Enable NMI, select upper VROM bank for both BG & sprite patterns
	lda #$98
	sta $2000


	rts
	



; NMI handler
nmi:
	; Save registers
	pha
	txa
	pha
	tya
	pha

	; Call sound driver
	jsr sound_driver_start	
	
	lda flipflop
	eor #1
	sta flipflop
	bne no_scroll_inc			; Only increase scrolling when the flipflop is zero
		inc hscroll			 
		bne no_wrap			; Did we reach the end of this nametable ?
			lda msg_nt		; ..yes - flip the nametable selector
			eor #1
			sta msg_nt
			jsr print_message	; Print the next 32 characters
		no_wrap:
	no_scroll_inc:

	; Select the correct nametable to be shown
	lda msg_nt
	ora #$98
	sta $2000

	lda hscroll
	sta $2005
	lda #$00
	sta $2005

	; Restore registers	
	pla
	tay
	pla
	tax
	pla
	
	rti



irq:
	rti
	

; DATA

; Character mappings for the serif8 font
character_set:
	.incbin "serif8.set"

; Nametable data. Divided into chunks of 256 bytes each
nametable_a:
	.incbin "nesmas-bg.nam1"
nametable_b:
	.incbin "nesmas-bg.nam2"
nametable_c:
	.incbin "nesmas-bg.nam3"

; Attribute table data. 48 bytes (256*192 pixels = 8*6 attributes)
attrtable:
	.incbin "nesmas-bg.atr"

sprite_data:
	; Sprite 0
	.db 23,38,32,248

	; Facial color
	.db 128,0,0,93
	.db 130,1,0,101
	.db 136,2,0,93
	.db 138,3,0,101
	.db 131,4,0,109
	.db 133,5,0,117
	.db 139,6,0,109
	.db 143,7,0,84
	.db 141,8,0,117
	.db 151,9,0,87
	.db 147,10,0,109
	.db 143,11,0,92

	; Sack color
	.db 153,12,0,61
	.db 161,13,0,61
	.db 153,14,0,69
	.db 165,15,0,70
	.db 173,16,0,75
	.db 181,17,0,77
	.db 189,18,0,78
	.db 173,19,0,83
	.db 187,20,0,85
	.db 187,21,0,93
	.db 188,22,0,101
	.db 188,23,0,109

	.db 190,24,0,117
	.db 186,25,2,55
	.db 203,26,0,79
	.db 202,27,0,87
	
	.db 206,28,0,97
	.db 199,29,0,105
	.db 198,30,0,113
	
	; Tree #1	
	.db 171,31,1,11
	.db 179,32,1,11
	.db 171,33,1,19
	.db 179,34,1,19

	; Tree #2	
	.db 196,31,1,200
	.db 204,32,1,200
	.db 196,33,1,208
	.db 204,34,1,208

	; 'NESmas 03'	
	.db 223,111,3,80
	.db 223,112,3,88
	.db 223,113,3,96
	.db 223,114,3,104
	.db 223,115,3,112
	.db 223,116,3,120
	.db 223,117,3,136
	.db 223,118,3,144
	
	
message:
	.db "Giddyup! It's that time of year again..     Mic gives you "
	.db "this year's coolest christmas greeting.    Turn up the vo"
	.db "lume and feel the awesome power of the NES!    "
	.db "And maybe if you've been good, Santa will bring you a gr"
	.db "ey box this year :)   "
	.db "   The muzak you"
message_b:
	.db "'re listening to is Howard Blake's 'Walking in the air', hack"
	.db "ed together in MML.      Thumbs up to the people in #nesde"
	.db "v for helping me fix some issues with this demo.       And"
	.db " finally, merry NESmas and a happy 2004 to all of you!  "
	.db ":)                     "
	.db 0

	
palette:
	.db 12,21,32,13		; blue,red,white,black
	.db 12,55,32,13		; blue,beige,white,black
	.db 12,7,32,13		; blue,brown,white,black
	.db 12,33,32,13		; blue,light blue,white,black
	
	.db 12,7,40,55		; blue,brown,light orange,beige
	.db 12,13,26,6		; blue,black,green,brown
	.db 12,0,16,32		; blue,grey,light grey,white
	.db 12,39,40,56		; blue,orange,light orange,light yellow

	

; Setup interrupt vectors
 .bank 3
 .org  $fffa
	.dw   nmi
	.dw   reset
	.dw   irq



; CHR-ROM
 .bank 4
 .org $0000
	; Background image. 253 tiles
	.incbin "nesmas-bg.pat"
 
 
 	; Tile consisting only of color 3 (which is black in all BG palettes)
 	black_tile:
 		.db $ff,$ff,$ff,$ff
 		.db $ff,$ff,$ff,$ff
 		.db $ff,$ff,$ff,$ff
 		.db $ff,$ff,$ff,$ff


 .org $1000
 	; Various sprites (39)
 	.incbin "nesmas-sprites.pat"

	; Monochrome font (72)
 	.incbin "serif8.pat"

	; 3-color font (8)
 	.incbin "nesmas.pat"
 	
 	
	
    